@@ -25,7 +25,7 @@ $serv->on('receive', function (swoole_server $serv, $fd, $reactor_id, $data) {
 	echo "[#".$serv->worker_id."]\tClient[$fd]: $data\n";
 });
 
-$serv->on('Task', function (swoole_server $serv, $task_id, $from_id, $data) {
+$serv->on('Task', function (swoole_server $serv, $task_id, $reactor_id, $data) {
     //echo "#{$serv->worker_id}\tonTask: [PID={$serv->worker_pid}]: task_id=$task_id, data_len=".strlen($data).".".PHP_EOL;
 //    $serv->finish($data);
     return $data;
