@@ -1,7 +1,7 @@
 <?php
 $table = new swoole_table(1024);
 $table->column('fd', swoole_table::TYPE_INT);
-$table->column('from_id', swoole_table::TYPE_INT);
+$table->column('reactor_id', swoole_table::TYPE_INT);
 $table->column('data', swoole_table::TYPE_STRING, 64);
 $table->create();
 
@@ -9,12 +9,12 @@ $serv = new swoole_server('127.0.0.1', 9501);
 $serv->set(['dispatch_mode' => 1]);
 $serv->table = $table;
 
-$serv->on('connect', function($serv, $fd, $from_id){
+$serv->on('connect', function($serv, $fd, $reactor_id){
 	$info = $serv->connection_info($fd);
-	$serv->send($fd, "INFO: fd=$fd, from_id=$from_id, addr={$info['remote_ip']}:{$info['remote_port']}\n");
+	$serv->send($fd, "INFO: fd=$fd, reactor_id=$reactor_id, addr={$info['remote_ip']}:{$info['remote_port']}\n");
 });
 
-$serv->on('receive', function ($serv, $fd, $from_id, $data) {
+$serv->on('receive', function ($serv, $fd, $reactor_id, $data) {
 
 	$cmd = explode(" ", trim($data));
 
@@ -33,7 +33,7 @@ $serv->on('receive', function ($serv, $fd, $from_id, $data) {
 	//set
 	elseif ($cmd[0] == 'set')
 	{
-		$ret = $serv->table->set($fd, array('from_id' => $data, 'fd' => $fd, 'data' => $cmd[1]));
+		$ret = $serv->table->set($fd, array('reactor_id' => $data, 'fd' => $fd, 'data' => $cmd[1]));
 		if ($ret === false)
 		{
 			$serv->send($fd, "ERROR\n");
