@@ -98,18 +98,14 @@ int clock_gettime(clock_id_t which_clock, struct timespec *t);
 #endif
 #endif
 
-#if __APPLE__
-#define  daemon(nochdir, noclose)    swoole_daemon(nochdir, noclose)
-#endif
-
 /*----------------------------------------------------------------------------*/
 
 #define SWOOLE_MAJOR_VERSION      4
 #define SWOOLE_MINOR_VERSION      4
-#define SWOOLE_RELEASE_VERSION    8
+#define SWOOLE_RELEASE_VERSION    13
 #define SWOOLE_EXTRA_VERSION      ""
-#define SWOOLE_VERSION            "4.4.8"
-#define SWOOLE_VERSION_ID         40408
+#define SWOOLE_VERSION            "4.4.13"
+#define SWOOLE_VERSION_ID         40413
 #define SWOOLE_BUG_REPORT \
     "A bug occurred in Swoole-v" SWOOLE_VERSION ", please report it.\n"\
     "The Swoole developers probably don't know about it,\n"\
@@ -282,6 +278,8 @@ static sw_inline int sw_mem_equal(const void *v1, size_t s1, const void *v2, siz
 #define sw_strndup             strndup
 #endif
 
+#define SW_Z_BEST_SPEED 1
+
 /** always return less than size, zero termination  */
 size_t sw_snprintf(char *buf, size_t size, const char *format, ...);
 size_t sw_vsnprintf(char *buf, size_t size, const char *format, va_list args);
@@ -307,6 +305,34 @@ static sw_inline char* swoole_strndup(const char *s, size_t n)
     return p;
 }
 
+/* string equal */
+static sw_inline unsigned int swoole_streq(const char *str1, size_t len1, const char *str2, size_t len2)
+{
+    return (len1 == len2) && (strncmp(str1, str2, len1) == 0);
+}
+
+static sw_inline unsigned int swoole_strcaseeq(const char *str1, size_t len1, const char *str2, size_t len2)
+{
+    return (len1 == len2) && (strncasecmp(str1, str2, len1) == 0);
+}
+
+static sw_inline unsigned int swoole_strct(const char *pstr, size_t plen, const char *sstr, size_t slen)
+{
+    return (plen >= slen) && (strncmp(pstr, sstr, slen) == 0);
+}
+
+static sw_inline unsigned int swoole_strcasect(const char *pstr, size_t plen, const char *sstr, size_t slen)
+{
+    return (plen >= slen) && (strncasecmp(pstr, sstr, slen) == 0);
+}
+
+#define SW_STREQ(str, len, const_str)      swoole_streq(str, len, SW_STRL(const_str))
+#define SW_STRCASEEQ(str, len, const_str)  swoole_strcaseeq(str, len, SW_STRL(const_str))
+
+/* string contain */
+#define SW_STRCT(str, len, const_sub_str)      swoole_strct(str, len, SW_STRL(const_sub_str))
+#define SW_STRCASECT(str, len, const_sub_str)  swoole_strcasect(str, len, SW_STRL(const_sub_str))
+
 /*--------------------------------Constants------------------------------------*/
 enum swResult_code
 {
@@ -388,8 +414,10 @@ enum swGlobal_hook_type
 
 enum swFork_type
 {
-    SW_FORK_SPAWN   = 0,
-    SW_FORK_EXEC    = 1u << 2,
+    SW_FORK_SPAWN    = 0,
+    SW_FORK_EXEC     = 1 << 1,
+    SW_FORK_DAEMON   = 1 << 2,
+    SW_FORK_PRECHECK = 1 << 3,
 };
 
 //-------------------------------------------------------------------------------
@@ -429,74 +457,77 @@ enum swWorker_status
 //-------------------------------------------------------------------------------
 
 #define swInfo(str,...) \
-    if (SW_LOG_INFO >= SwooleG.log_level) {\
-        size_t _sw_error_len = sw_snprintf(sw_error,SW_ERROR_MSG_SIZE,str,##__VA_ARGS__);\
-        SwooleG.write_log(SW_LOG_INFO, sw_error, _sw_error_len);\
+    if (SW_LOG_INFO >= SwooleG.log_level) { \
+        size_t _sw_error_len = sw_snprintf(sw_error,SW_ERROR_MSG_SIZE,str,##__VA_ARGS__); \
+        SwooleG.write_log(SW_LOG_INFO, sw_error, _sw_error_len); \
     }
 
 #define swNotice(str,...) \
-    if (SW_LOG_NOTICE >= SwooleG.log_level) {\
-        size_t _sw_error_len = sw_snprintf(sw_error,SW_ERROR_MSG_SIZE,str,##__VA_ARGS__);\
-        SwooleG.write_log(SW_LOG_NOTICE, sw_error, _sw_error_len);\
+    if (SW_LOG_NOTICE >= SwooleG.log_level) { \
+        size_t _sw_error_len = sw_snprintf(sw_error,SW_ERROR_MSG_SIZE,str,##__VA_ARGS__); \
+        SwooleG.write_log(SW_LOG_NOTICE, sw_error, _sw_error_len); \
     }
 
 #define swSysNotice(str,...) \
-    do{\
-        SwooleG.error = errno;\
-        if (SW_LOG_ERROR >= SwooleG.log_level) {\
-            size_t _sw_error_len = sw_snprintf(sw_error,SW_ERROR_MSG_SIZE,"%s(:%d): " str ", Error: %s[%d]",__func__,__LINE__,##__VA_ARGS__,strerror(errno),errno);\
-            SwooleG.write_log(SW_LOG_NOTICE, sw_error, _sw_error_len);\
-        }\
+    do{ \
+        SwooleG.error = errno; \
+        if (SW_LOG_ERROR >= SwooleG.log_level) { \
+            size_t _sw_error_len = sw_snprintf(sw_error,SW_ERROR_MSG_SIZE,"%s(:%d): " str ", Error: %s[%d]",__func__,__LINE__,##__VA_ARGS__,strerror(errno),errno); \
+            SwooleG.write_log(SW_LOG_NOTICE, sw_error, _sw_error_len); \
+        } \
     } while(0)
 
 #define swWarn(str,...) \
-    do{\
-        if (SW_LOG_WARNING >= SwooleG.log_level) {\
-            size_t _sw_error_len = sw_snprintf(sw_error,SW_ERROR_MSG_SIZE,"%s: " str,__func__,##__VA_ARGS__);\
-            SwooleG.write_log(SW_LOG_WARNING, sw_error, _sw_error_len);\
-        }\
+    do{ \
+        if (SW_LOG_WARNING >= SwooleG.log_level) { \
+            size_t _sw_error_len = sw_snprintf(sw_error,SW_ERROR_MSG_SIZE,"%s: " str,__func__,##__VA_ARGS__); \
+            SwooleG.write_log(SW_LOG_WARNING, sw_error, _sw_error_len); \
+        } \
     } while(0)
 
 #define swSysWarn(str,...) \
-    do{\
-        SwooleG.error = errno;\
-        if (SW_LOG_ERROR >= SwooleG.log_level) {\
-            size_t _sw_error_len = sw_snprintf(sw_error,SW_ERROR_MSG_SIZE,"%s(:%d): " str ", Error: %s[%d]",__func__,__LINE__,##__VA_ARGS__,strerror(errno),errno);\
-            SwooleG.write_log(SW_LOG_WARNING, sw_error, _sw_error_len);\
-        }\
+    do{ \
+        SwooleG.error = errno; \
+        if (SW_LOG_ERROR >= SwooleG.log_level) { \
+            size_t _sw_error_len = sw_snprintf(sw_error,SW_ERROR_MSG_SIZE,"%s(:%d): " str ", Error: %s[%d]",__func__,__LINE__,##__VA_ARGS__,strerror(errno),errno); \
+            SwooleG.write_log(SW_LOG_WARNING, sw_error, _sw_error_len); \
+        } \
     } while(0)
 
 #define swError(str,...) \
-    do{\
-        size_t _sw_error_len = sw_snprintf(sw_error, SW_ERROR_MSG_SIZE, str, ##__VA_ARGS__);\
-        SwooleG.write_log(SW_LOG_ERROR, sw_error, _sw_error_len);\
-        exit(1);\
+    do{ \
+        size_t _sw_error_len = sw_snprintf(sw_error, SW_ERROR_MSG_SIZE, str, ##__VA_ARGS__); \
+        SwooleG.write_log(SW_LOG_ERROR, sw_error, _sw_error_len); \
+        exit(1); \
     } while(0)
 
 #define swSysError(str,...) \
-    do{\
-        size_t _sw_error_len = sw_snprintf(sw_error,SW_ERROR_MSG_SIZE,"%s(:%d): " str ", Error: %s[%d]",__func__,__LINE__,##__VA_ARGS__,strerror(errno),errno);\
-        SwooleG.write_log(SW_LOG_ERROR, sw_error, _sw_error_len);\
-        exit(1);\
+    do{ \
+        size_t _sw_error_len = sw_snprintf(sw_error,SW_ERROR_MSG_SIZE,"%s(:%d): " str ", Error: %s[%d]",__func__,__LINE__,##__VA_ARGS__,strerror(errno),errno); \
+        SwooleG.write_log(SW_LOG_ERROR, sw_error, _sw_error_len); \
+        exit(1); \
     } while(0)
 
 #define swFatalError(code, str,...) \
-        SwooleG.fatal_error(code, str, ##__VA_ARGS__)
+    do { \
+        SwooleG.fatal_error(code, str, ##__VA_ARGS__); \
+        abort(); \
+    } while (0)
 
 #define swoole_error_log(level, __errno, str, ...) \
-    do{\
-        SwooleG.error = __errno;\
-        if (level >= SwooleG.log_level){\
-            size_t _sw_error_len = sw_snprintf(sw_error, SW_ERROR_MSG_SIZE, "%s (ERRNO %d): " str,__func__,__errno,##__VA_ARGS__);\
-            SwooleG.write_log(level, sw_error, _sw_error_len);\
-        }\
+    do{ \
+        SwooleG.error = __errno; \
+        if (level >= SwooleG.log_level){ \
+            size_t _sw_error_len = sw_snprintf(sw_error, SW_ERROR_MSG_SIZE, "%s (ERRNO %d): " str,__func__,__errno,##__VA_ARGS__); \
+            SwooleG.write_log(level, sw_error, _sw_error_len); \
+        } \
     } while(0)
 
 #ifdef SW_DEBUG
 #define swDebug(str,...) \
-    if (SW_LOG_DEBUG >= SwooleG.log_level) {\
-        size_t _sw_error_len = sw_snprintf(sw_error, SW_ERROR_MSG_SIZE, "%s(:%d): " str, __func__, __LINE__, ##__VA_ARGS__);\
-        SwooleG.write_log(SW_LOG_DEBUG, sw_error, _sw_error_len);\
+    if (SW_LOG_DEBUG >= SwooleG.log_level) { \
+        size_t _sw_error_len = sw_snprintf(sw_error, SW_ERROR_MSG_SIZE, "%s(:%d): " str, __func__, __LINE__, ##__VA_ARGS__); \
+        SwooleG.write_log(SW_LOG_DEBUG, sw_error, _sw_error_len); \
     }
 
 #define swHexDump(data, length) \
@@ -717,6 +748,9 @@ typedef struct _swConnection
     uint8_t http_upgrade;
 #ifdef SW_USE_HTTP2
     uint8_t http2_stream;
+#endif
+#ifdef SW_HAVE_ZLIB
+    uint8_t websocket_compression;
 #endif
     //--------------------------------------------------------------
     /**
@@ -1497,6 +1531,9 @@ void swoole_redirect_stdout(int new_fd);
 int swoole_shell_exec(const char *command, pid_t *pid, uint8_t get_error_stream);
 int swoole_daemon(int nochdir, int noclose);
 
+SW_API const char* swoole_version(void);
+SW_API int swoole_version_id(void);
+
 SW_API int swoole_add_function(const char *name, void* func);
 SW_API void* swoole_get_function(const char *name, uint32_t length);
 SW_API int swoole_add_hook(enum swGlobal_hook_type type, swCallback func, int push_back);
@@ -1524,9 +1561,10 @@ static sw_inline uint64_t swoole_ntoh64(uint64_t net)
     uint32_t high, low;
 
     low = net & 0xFFFFFFFF;
-    high = (net >> 32) & 0xFFFFFFFF;
+    high = net >> 32;
     low = ntohl(low);
     high = ntohl(high);
+
     ret = low;
     ret <<= 32;
     ret |= high;
@@ -1626,10 +1664,6 @@ struct _swReactor
     uint32_t start :1;
     uint32_t once :1;
     uint32_t wait_exit :1;
-    /**
-     * disable accept new connection
-     */
-    uint32_t disable_accept :1;
     /**
      * callback signal
      */
@@ -1675,7 +1709,6 @@ struct _swReactor
     void (*onFinish)(swReactor *);
     void (*onBegin)(swReactor *);
 
-    void (*enable_accept)(swReactor *);
     int (*is_empty)(swReactor *);
 
     int (*write)(swReactor *, int, const void *, int);
@@ -2362,6 +2395,7 @@ typedef struct
      */
     uint32_t socket_buffer_size;
     swArray *socket_array;
+    double socket_send_timeout;
 
     swServer *serv;
 
