@@ -310,26 +310,15 @@ enum process_pipe_type
 class process
 {
 public:
-    php_swoole_fci *func;
-    zend_object *zsocket;
+    zend_object *zsocket = nullptr;
     enum process_pipe_type pipe_type;
     bool enable_coroutine;
 
-    process()
-    {
-        func = nullptr;
-        zsocket = nullptr;
-        pipe_type = PIPE_TYPE_NONE;
-        enable_coroutine = false;
-    }
+    process(enum process_pipe_type pipe_type, bool enable_coroutine) :
+        pipe_type(pipe_type), enable_coroutine(enable_coroutine) { }
 
     ~process()
     {
-        if (func)
-        {
-            sw_zend_fci_cache_discard(&func->fci_cache);
-            efree(func);
-        }
         if (zsocket)
         {
             OBJ_RELEASE(zsocket);
