@@ -14,7 +14,6 @@
   +----------------------------------------------------------------------+
 */
 
-#include "swoole.h"
 #include "server.h"
 #include "client.h"
 
@@ -29,10 +28,9 @@ static int swProcessPool_worker_loop_ex(swProcessPool *pool, swWorker *worker);
 
 static void swProcessPool_free(swProcessPool *pool);
 
-
-static void swProcessPool_killTimeout(swTimer *timer, swTimer_node *tnode)
+static void swProcessPool_kill_timeout_worker(swTimer *timer, swTimer_node *tnode)
 {
-    int i;
+    uint32_t i;
     pid_t reload_worker_pid = 0;
     swProcessPool *pool = (swProcessPool *)tnode->data;
     pool->reloading = 0;
@@ -42,11 +40,11 @@ static void swProcessPool_killTimeout(swTimer *timer, swTimer_node *tnode)
         if (i >= pool->reload_worker_i)
         {
             reload_worker_pid = pool->reload_workers[i].pid;
-            if (swKill(reload_worker_pid, 0) == -1)
+            if (swoole_kill(reload_worker_pid, 0) == -1)
             {
                 continue;
             }
-            if (swKill(reload_worker_pid, SIGKILL) < 0)
+            if (swoole_kill(reload_worker_pid, SIGKILL) < 0)
             {
                 swSysWarn("swKill(%d, SIGKILL) [%d] failed", pool->reload_workers[i].pid, i);
             }
@@ -63,15 +61,17 @@ static void swProcessPool_killTimeout(swTimer *timer, swTimer_node *tnode)
 /**
  * Process manager
  */
-int swProcessPool_create(swProcessPool *pool, int worker_num, int max_request, key_t msgqueue_key, int ipc_mode)
+int swProcessPool_create(swProcessPool *pool, uint32_t worker_num, key_t msgqueue_key, int ipc_mode)
 {
     bzero(pool, sizeof(swProcessPool));
 
     int i;
 
     pool->worker_num = worker_num;
-    pool->max_request = max_request;
 
+    /**
+     * Shared memory is used here
+     */
     pool->workers = SwooleG.memory_pool->alloc(SwooleG.memory_pool, worker_num * sizeof(swWorker));
     if (pool->workers == NULL)
     {
@@ -79,7 +79,6 @@ int swProcessPool_create(swProcessPool *pool, int worker_num, int max_request, k
         return SW_ERR;
     }
 
-#ifndef _WIN32
     if (ipc_mode == SW_IPC_MSGQUEUE)
     {
         pool->use_msgqueue = 1;
@@ -119,19 +118,17 @@ int swProcessPool_create(swProcessPool *pool, int worker_num, int max_request, k
             pool->workers[i].pipe_object = pipe;
         }
     }
-	else
-#endif
-	if (ipc_mode == SW_IPC_SOCKET)
-	{
-		pool->use_socket = 1;
-		pool->stream = sw_malloc(sizeof(swStreamInfo));
-		if (pool->stream == NULL)
-		{
-			swWarn("malloc[2] failed");
-			return SW_ERR;
-		}
-		bzero(pool->stream, sizeof(swStreamInfo));
-	}
+    else if (ipc_mode == SW_IPC_SOCKET)
+    {
+        pool->use_socket = 1;
+        pool->stream = sw_malloc(sizeof(swStreamInfo));
+        if (pool->stream == NULL)
+        {
+            swWarn("malloc[2] failed");
+            return SW_ERR;
+        }
+        bzero(pool->stream, sizeof(swStreamInfo));
+    }
     else
     {
         ipc_mode = SW_IPC_NONE;
@@ -209,7 +206,7 @@ int swProcessPool_start(swProcessPool *pool)
         return SW_ERR;
     }
 
-    int i;
+    uint32_t i;
     pool->started = 1;
 
     for (i = 0; i < pool->worker_num; i++)
@@ -237,7 +234,7 @@ static sw_inline int swProcessPool_schedule(swProcessPool *pool)
         return 0;
     }
 
-    int i, target_worker_id = 0;
+    uint32_t i, target_worker_id = 0;
 
     for (i = 0; i < pool->worker_num + 1; i++)
     {
@@ -357,16 +354,17 @@ int swProcessPool_dispatch_blocking(swProcessPool *pool, swEventData *data, int
 
 void swProcessPool_shutdown(swProcessPool *pool)
 {
-    int i, status;
+    uint32_t i;
+    int status;
     swWorker *worker;
     SwooleG.running = 0;
 
-	swSignal_none();
+    swSignal_none();
     //concurrent kill
     for (i = 0; i < pool->worker_num; i++)
     {
         worker = &pool->workers[i];
-        if (swKill(worker->pid, SIGTERM) < 0)
+        if (swoole_kill(worker->pid, SIGTERM) < 0)
         {
             swSysWarn("swKill(%d) failed", worker->pid);
             continue;
@@ -375,7 +373,7 @@ void swProcessPool_shutdown(swProcessPool *pool)
     for (i = 0; i < pool->worker_num; i++)
     {
         worker = &pool->workers[i];
-        if (swWaitpid(worker->pid, &status, 0) < 0)
+        if (swoole_waitpid(worker->pid, &status, 0) < 0)
         {
             swSysWarn("waitpid(%d) failed", worker->pid);
         }
@@ -386,7 +384,7 @@ void swProcessPool_shutdown(swProcessPool *pool)
 
 pid_t swProcessPool_spawn(swProcessPool *pool, swWorker *worker)
 {
-    pid_t pid = swoole_fork();
+    pid_t pid = swoole_fork(0);
     int ret_code = 0;
 
     switch (pid)
@@ -444,18 +442,20 @@ int swProcessPool_get_max_request(swProcessPool *pool)
     else
     {
         task_n = pool->max_request;
-        if (pool->max_request > 10)
+        if (pool->max_request_grace > 0)
         {
-            int n = swoole_system_random(1, pool->max_request / 2);
-            if (n > 0)
-            {
-                task_n += n;
-            }
+            task_n += swoole_system_random(1, pool->max_request_grace);
         }
     }
     return task_n;
 }
 
+void swProcessPool_set_max_request(swProcessPool *pool, uint32_t max_request, uint32_t max_request_grace)
+{
+    pool->max_request = max_request;
+    pool->max_request_grace = max_request_grace;
+}
+
 static int swProcessPool_worker_loop(swProcessPool *pool, swWorker *worker)
 {
     struct
@@ -475,7 +475,7 @@ static int swProcessPool_worker_loop(swProcessPool *pool, swWorker *worker)
     /**
      * Use from_fd save the task_worker->id
      */
-    out.buf.info.from_fd = worker->id;
+    out.buf.info.server_fd = worker->id;
 
     if (pool->dispatch_mode == SW_DISPATCH_QUEUE)
     {
@@ -538,10 +538,11 @@ static int swProcessPool_worker_loop(swProcessPool *pool, swWorker *worker)
          */
         if (n < 0)
         {
-            if (errno == EINTR && SwooleG.signal_alarm)
+            if (errno == EINTR && SwooleWG.signal_alarm && SwooleTG.timer)
             {
-                alarm_handler: SwooleG.signal_alarm = 0;
-                swTimer_select(&SwooleG.timer);
+                _alarm_handler:
+                SwooleWG.signal_alarm = 0;
+                swTimer_select(SwooleTG.timer);
             }
             continue;
         }
@@ -550,11 +551,8 @@ static int swProcessPool_worker_loop(swProcessPool *pool, swWorker *worker)
          * do task
          */
         worker->status = SW_WORKER_BUSY;
-        worker->request_time = time(NULL);
         ret = pool->onTask(pool, &out.buf);
         worker->status = SW_WORKER_IDLE;
-        worker->request_time = 0;
-        worker->traced = 0;
 
         if (pool->use_socket && pool->stream->last_connection > 0)
         {
@@ -567,9 +565,9 @@ static int swProcessPool_worker_loop(swProcessPool *pool, swWorker *worker)
         /**
          * timer
          */
-        if (SwooleG.signal_alarm)
+        if (SwooleWG.signal_alarm)
         {
-            goto alarm_handler;
+            goto _alarm_handler;
         }
 
         if (ret >= 0 && !worker_task_always)
@@ -665,7 +663,8 @@ static int swProcessPool_worker_loop_ex(swProcessPool *pool, swWorker *worker)
             }
             if (swSocket_recv_blocking(fd, pool->packet_buffer, n, MSG_WAITALL) <= 0)
             {
-                _close: close(fd);
+                _close:
+                close(fd);
                 continue;
             }
             data = pool->packet_buffer;
@@ -686,10 +685,11 @@ static int swProcessPool_worker_loop_ex(swProcessPool *pool, swWorker *worker)
          */
         if (n < 0)
         {
-            if (errno == EINTR && SwooleG.signal_alarm)
+            if (errno == EINTR && SwooleWG.signal_alarm && SwooleTG.timer)
             {
-                alarm_handler: SwooleG.signal_alarm = 0;
-                swTimer_select(&SwooleG.timer);
+                _alarm_handler:
+                SwooleWG.signal_alarm = 0;
+                swTimer_select(SwooleTG.timer);
             }
             continue;
         }
@@ -713,9 +713,9 @@ static int swProcessPool_worker_loop_ex(swProcessPool *pool, swWorker *worker)
         /**
          * timer
          */
-        if (SwooleG.signal_alarm)
+        if (SwooleWG.signal_alarm)
         {
-            goto alarm_handler;
+            goto _alarm_handler;
         }
     }
     return SW_OK;
@@ -733,7 +733,6 @@ int swProcessPool_add_worker(swProcessPool *pool, swWorker *worker)
 int swProcessPool_wait(swProcessPool *pool)
 {
     int pid, new_pid;
-//    int reload_worker_i = 0;
     pid_t reload_worker_pid = 0;
     int ret;
     int status;
@@ -748,10 +747,10 @@ int swProcessPool_wait(swProcessPool *pool)
     while (SwooleG.running)
     {
         pid = wait(&status);
-        if (SwooleG.signal_alarm == 1)
+        if (SwooleWG.signal_alarm && SwooleTG.timer)
         {
-            SwooleG.signal_alarm = 0;
-            swTimer_select(&SwooleG.timer);
+            SwooleWG.signal_alarm = 0;
+            swTimer_select(SwooleTG.timer);
         }
         if (pid < 0)
         {
@@ -776,10 +775,10 @@ int swProcessPool_wait(swProcessPool *pool)
                     memcpy(pool->reload_workers, pool->workers, sizeof(swWorker) * pool->worker_num);
                     if (pool->max_wait_time)
                     {
-                        swTimer_add(&SwooleG.timer, (long) (pool->max_wait_time * 1000), 0, pool, swProcessPool_killTimeout);
+                        swoole_timer_add((long) (pool->max_wait_time * 1000), SW_FALSE, swProcessPool_kill_timeout_worker, pool);
                     }
                 }
-                goto kill_worker;
+                goto _kill_worker;
             }
         }
 
@@ -820,7 +819,8 @@ int swProcessPool_wait(swProcessPool *pool)
             }
         }
         //reload worker
-        kill_worker: if (pool->reloading == 1)
+        _kill_worker:
+        if (pool->reloading == 1)
         {
             //reload finish
             if (pool->reload_worker_i >= pool->worker_num)
@@ -829,13 +829,13 @@ int swProcessPool_wait(swProcessPool *pool)
                 continue;
             }
             reload_worker_pid = pool->reload_workers[pool->reload_worker_i].pid;
-            ret = swKill(reload_worker_pid, SIGTERM);
+            ret = swoole_kill(reload_worker_pid, SIGTERM);
             if (ret < 0)
             {
                 if (errno == ECHILD)
                 {
                     pool->reload_worker_i++;
-                    goto kill_worker;
+                    goto _kill_worker;
                 }
                 swSysWarn("[Manager]swKill(%d) failed", pool->reload_workers[pool->reload_worker_i].pid);
                 continue;
@@ -843,12 +843,13 @@ int swProcessPool_wait(swProcessPool *pool)
         }
     }
     sw_free(pool->reload_workers);
+    pool->reload_workers = NULL;
     return SW_OK;
 }
 
 static void swProcessPool_free(swProcessPool *pool)
 {
-    int i;
+    uint32_t i;
     swPipe *_pipe;
 
     if (pool->pipes)
@@ -884,6 +885,11 @@ static void swProcessPool_free(swProcessPool *pool)
         sw_free(pool->stream);
     }
 
+    if (pool->packet_buffer)
+    {
+        sw_free(pool->packet_buffer);
+    }
+
     if (pool->map)
     {
         swHashMap_free(pool->map);
