@@ -63,7 +63,6 @@ int swReactorPoll_create(swReactor *reactor, int max_fd_num)
     }
     object->max_fd_num = max_fd_num;
     reactor->max_event_num = max_fd_num;
-    bzero(reactor->handle, sizeof(reactor->handle));
     reactor->object = object;
     //binding method
     reactor->add = swReactorPoll_add;
@@ -119,7 +118,6 @@ static int swReactorPoll_add(swReactor *reactor, int fd, int fdtype)
         object->events[cur].events |= POLLHUP;
     }
 
-    reactor->event_num++;
     return SW_OK;
 }
 
@@ -163,11 +161,9 @@ static int swReactorPoll_del(swReactor *reactor, int fd)
     {
         if (object->events[i].fd == fd)
         {
-            uint32_t old_num = reactor->event_num;
-            reactor->event_num = reactor->event_num <= 0 ? 0 : reactor->event_num - 1;
-            for (; i < old_num; i++)
+            for (; i < reactor->event_num; i++)
             {
-                if (i == old_num)
+                if (i == reactor->event_num)
                 {
                     object->fds[i].fdtype = 0;
                     object->events[i].fd = 0;
@@ -188,9 +184,9 @@ static int swReactorPoll_del(swReactor *reactor, int fd)
 
 static int swReactorPoll_wait(swReactor *reactor, struct timeval *timeo)
 {
-    swReactorPoll *object = reactor->object;
+    swReactorPoll *object = (swReactorPoll *) reactor->object;
     swEvent event;
-    swReactor_handle handle;
+    swReactor_handler handler;
 
     int ret, i;
 
@@ -224,12 +220,12 @@ static int swReactorPoll_wait(swReactor *reactor, struct timeval *timeo)
             }
             else
             {
-            	goto _continue;
+                goto _continue;
             }
         }
         else if (ret == 0)
         {
-            if (reactor->onTimeout != NULL)
+            if (reactor->onTimeout)
             {
                 reactor->onTimeout(reactor);
             }
@@ -240,16 +236,16 @@ static int swReactorPoll_wait(swReactor *reactor, struct timeval *timeo)
             for (i = 0; i < reactor->event_num; i++)
             {
                 event.fd = object->events[i].fd;
-                event.from_id = reactor->id;
+                event.reactor_id = reactor->id;
                 event.type = object->fds[i].fdtype;
                 event.socket = swReactor_get(reactor, event.fd);
 
-                swTrace("Event: fd=%d|from_id=%d|type=%d", event.fd, reactor->id, object->fds[i].fdtype);
+                swTrace("Event: fd=%d|reactor_id=%d|type=%d", event.fd, reactor->id, object->fds[i].fdtype);
                 //in
                 if ((object->events[i].revents & POLLIN) && !event.socket->removed)
                 {
-                    handle = swReactor_getHandle(reactor, SW_EVENT_READ, event.type);
-                    ret = handle(reactor, &event);
+                    handler = swReactor_get_handler(reactor, SW_EVENT_READ, event.type);
+                    ret = handler(reactor, &event);
                     if (ret < 0)
                     {
                         swSysWarn("poll[POLLIN] handler failed. fd=%d", event.fd);
@@ -258,8 +254,8 @@ static int swReactorPoll_wait(swReactor *reactor, struct timeval *timeo)
                 //out
                 if ((object->events[i].revents & POLLOUT) && !event.socket->removed)
                 {
-                    handle = swReactor_getHandle(reactor, SW_EVENT_WRITE, event.type);
-                    ret = handle(reactor, &event);
+                    handler = swReactor_get_handler(reactor, SW_EVENT_WRITE, event.type);
+                    ret = handler(reactor, &event);
                     if (ret < 0)
                     {
                         swSysWarn("poll[POLLOUT] handler failed. fd=%d", event.fd);
@@ -273,8 +269,8 @@ static int swReactorPoll_wait(swReactor *reactor, struct timeval *timeo)
                     {
                         continue;
                     }
-                    handle = swReactor_getHandle(reactor, SW_EVENT_ERROR, event.type);
-                    ret = handle(reactor, &event);
+                    handler = swReactor_get_handler(reactor, SW_EVENT_ERROR, event.type);
+                    ret = handler(reactor, &event);
                     if (ret < 0)
                     {
                         swSysWarn("poll[POLLERR] handler failed. fd=%d", event.fd);
@@ -282,11 +278,12 @@ static int swReactorPoll_wait(swReactor *reactor, struct timeval *timeo)
                 }
                 if (!event.socket->removed && (event.socket->events & SW_EVENT_ONCE))
                 {
-                    reactor->del(reactor, event.fd);
+                    swReactorPoll_del(reactor, event.fd);
                 }
             }
         }
-        _continue: if (reactor->onFinish != NULL)
+        _continue:
+        if (reactor->onFinish)
         {
             reactor->onFinish(reactor);
         }
