@@ -57,7 +57,6 @@ int swReactorSelect_create(swReactor *reactor)
 
     object->fds = NULL;
     object->maxfd = 0;
-    bzero(reactor->handle, sizeof(reactor->handle));
     reactor->object = object;
     //binding method
     reactor->add = swReactorSelect_add;
@@ -101,7 +100,6 @@ int swReactorSelect_add(swReactor *reactor, int fd, int fdtype)
     ev->fd = fd;
     ev->fdtype = fdtype;
     LL_APPEND(object->fds, ev);
-    reactor->event_num++;
     if (fd > object->maxfd)
     {
         object->maxfd = fd;
@@ -130,7 +128,6 @@ int swReactorSelect_del(swReactor *reactor, int fd)
     SW_FD_CLR(fd, &object->rfds);
     SW_FD_CLR(fd, &object->wfds);
     SW_FD_CLR(fd, &object->efds);
-    reactor->event_num = reactor->event_num <= 0 ? 0 : reactor->event_num - 1;
     sw_free(s_ev);
     swReactor_del(reactor, fd);
     return SW_OK;
@@ -159,7 +156,7 @@ int swReactorSelect_wait(swReactor *reactor, struct timeval *timeo)
     swFdList_node *ev;
     swFdList_node *tmp;
     swEvent event;
-    swReactor_handle handle;
+    swReactor_handler handler;
     struct timeval timeout;
     int ret;
 
@@ -230,12 +227,12 @@ int swReactorSelect_wait(swReactor *reactor, struct timeval *timeo)
             }
             else
             {
-            	goto _continue;
+                goto _continue;
             }
         }
         else if (ret == 0)
         {
-            if (reactor->onTimeout != NULL)
+            if (reactor->onTimeout)
             {
                 reactor->onTimeout(reactor);
             }
@@ -246,15 +243,15 @@ int swReactorSelect_wait(swReactor *reactor, struct timeval *timeo)
             LL_FOREACH_SAFE(object->fds, ev, tmp)
             {
                 event.fd = ev->fd;
-                event.from_id = reactor->id;
+                event.reactor_id = reactor->id;
                 event.type = swReactor_fdtype(ev->fdtype);
                 event.socket = swReactor_get(reactor, event.fd);
 
                 //read
                 if (SW_FD_ISSET(event.fd, &(object->rfds)) && !event.socket->removed)
                 {
-                    handle = swReactor_getHandle(reactor, SW_EVENT_READ, event.type);
-                    ret = handle(reactor, &event);
+                    handler = swReactor_get_handler(reactor, SW_EVENT_READ, event.type);
+                    ret = handler(reactor, &event);
                     if (ret < 0)
                     {
                         swSysWarn("[Reactor#%d] select event[type=READ, fd=%d] handler fail", reactor->id, event.fd);
@@ -263,8 +260,8 @@ int swReactorSelect_wait(swReactor *reactor, struct timeval *timeo)
                 //write
                 if (SW_FD_ISSET(event.fd, &(object->wfds)) && !event.socket->removed)
                 {
-                    handle = swReactor_getHandle(reactor, SW_EVENT_WRITE, event.type);
-                    ret = handle(reactor, &event);
+                    handler = swReactor_get_handler(reactor, SW_EVENT_WRITE, event.type);
+                    ret = handler(reactor, &event);
                     if (ret < 0)
                     {
                         swSysWarn("[Reactor#%d] select event[type=WRITE, fd=%d] handler fail", reactor->id, event.fd);
@@ -273,8 +270,8 @@ int swReactorSelect_wait(swReactor *reactor, struct timeval *timeo)
                 //error
                 if (SW_FD_ISSET(event.fd, &(object->efds)) && !event.socket->removed)
                 {
-                    handle = swReactor_getHandle(reactor, SW_EVENT_ERROR, event.type);
-                    ret = handle(reactor, &event);
+                    handler = swReactor_get_handler(reactor, SW_EVENT_ERROR, event.type);
+                    ret = handler(reactor, &event);
                     if (ret < 0)
                     {
                         swSysWarn("[Reactor#%d] select event[type=ERROR, fd=%d] handler fail", reactor->id, event.fd);
@@ -282,11 +279,12 @@ int swReactorSelect_wait(swReactor *reactor, struct timeval *timeo)
                 }
                 if (!event.socket->removed && (event.socket->events & SW_EVENT_ONCE))
                 {
-                    reactor->del(reactor, event.fd);
+                    swReactorSelect_del(reactor, event.fd);
                 }
             }
         }
-        _continue: if (reactor->onFinish != NULL)
+        _continue:
+        if (reactor->onFinish)
         {
             reactor->onFinish(reactor);
         }
