@@ -21,6 +21,8 @@
 #include "thirdparty/swoole_http_parser.h"
 #include "thirdparty/multipart_parser.h"
 
+#include <unordered_map>
+
 #ifdef SW_HAVE_ZLIB
 #include <zlib.h>
 #define SW_ZLIB_ENCODING_RAW        -0xf
@@ -110,13 +112,16 @@ struct http_context
     uint32_t completed :1;
     uint32_t end :1;
     uint32_t send_header :1;
-#ifdef SW_HAVE_ZLIB
+#ifdef SW_HAVE_COMPRESSION
     uint32_t enable_compression :1;
     uint32_t accept_compression :1;
 #endif
     uint32_t chunk :1;
     uint32_t keepalive :1;
     uint32_t websocket :1;
+#ifdef SW_HAVE_ZLIB
+    uint32_t websocket_compression :1;
+#endif
     uint32_t upgrade :1;
     uint32_t detached :1;
     uint32_t parse_cookie :1;
@@ -124,7 +129,7 @@ struct http_context
     uint32_t parse_files :1;
     uint32_t co_socket :1;
 
-#ifdef SW_HAVE_ZLIB
+#ifdef SW_HAVE_COMPRESSION
     int8_t compression_level;
     int8_t compression_method;
 #endif
@@ -156,23 +161,64 @@ struct http_context
     bool (*close)(http_context* ctx);
 };
 
+#ifdef SW_USE_HTTP2
+class http2_stream
+{
+public:
+    http_context* ctx;
+    // uint8_t priority; // useless now
+    uint32_t id;
+    // flow control
+    uint32_t send_window;
+    uint32_t recv_window;
+
+    http2_stream(int _fd, uint32_t _id);
+    ~http2_stream();
+
+    void reset(uint32_t error_code);
+};
+
+class http2_session
+{
+public:
+    int fd;
+    std::unordered_map<uint32_t, http2_stream*> streams;
+
+    nghttp2_hd_inflater *inflater = nullptr;
+    nghttp2_hd_deflater *deflater = nullptr;
+
+    uint32_t header_table_size;
+    uint32_t send_window;
+    uint32_t recv_window;
+    uint32_t max_concurrent_streams;
+    uint32_t max_frame_size;
+
+    http_context *default_ctx = nullptr;
+    void *private_data = nullptr;
+
+    void (*handle)(http2_session *, http2_stream *) = nullptr;
+
+    http2_session(int _fd);
+    ~http2_session();
+};
+#endif
+
 extern zend_class_entry *swoole_http_server_ce;
 extern zend_class_entry *swoole_http_request_ce;
 extern zend_class_entry *swoole_http_response_ce;
 
 extern swString *swoole_http_buffer;
 extern swString *swoole_http_form_data_buffer;
-#ifdef SW_HAVE_ZLIB
+#ifdef SW_HAVE_COMPRESSION
 extern swString *swoole_zlib_buffer;
 #endif
 
-#define http_strncasecmp(const_str, at, length) \
-    ((length >= sizeof(const_str)-1) && \
-    (strncasecmp(at, ZEND_STRL(const_str)) == 0))
-
 http_context* swoole_http_context_new(int fd);
-http_context* swoole_http_context_get(zval *zobject, const bool check_end);
+http_context* php_swoole_http_request_get_and_check_context(zval *zobject);
+http_context* php_swoole_http_response_get_and_check_context(zval *zobject);
 void swoole_http_context_free(http_context *ctx);
+void swoole_http_context_copy(http_context *src, http_context *dst);
+
 static sw_inline zval* swoole_http_init_and_read_property(zend_class_entry *ce, zval *zobject, zval** zproperty_store_pp, const char *name, size_t name_len)
 {
     if (UNEXPECTED(!*zproperty_store_pp))
@@ -187,19 +233,24 @@ static sw_inline zval* swoole_http_init_and_read_property(zend_class_entry *ce,
 }
 int swoole_http_parse_form_data(http_context *ctx, const char *boundary_str, int boundary_len);
 void swoole_http_parse_cookie(zval *array, const char *at, size_t length);
-
 void swoole_http_server_init_context(swServer *serv, http_context *ctx);
 
+http_context * php_swoole_http_request_get_context(zval *zobject);
+void php_swoole_http_request_set_context(zval *zobject, http_context *context);
+http_context * php_swoole_http_response_get_context(zval *zobject);
+void php_swoole_http_response_set_context(zval *zobject, http_context *context);
 size_t swoole_http_requset_parse(http_context *ctx, const char *data, size_t length);
 
 bool swoole_http_response_set_header(http_context *ctx, const char *k, size_t klen, const char *v, size_t vlen, bool ucwords);
 void swoole_http_response_end(http_context *ctx, zval *zdata, zval *return_value);
 
-#ifdef SW_HAVE_ZLIB
+#ifdef SW_HAVE_COMPRESSION
 int swoole_http_response_compress(swString *body, int method, int level);
 void swoole_http_get_compression_method(http_context *ctx, const char *accept_encoding, size_t length);
 const char* swoole_http_get_content_encoding(http_context *ctx);
+#endif
 
+#ifdef SW_HAVE_ZLIB
 static sw_inline voidpf php_zlib_alloc(voidpf opaque, uInt items, uInt size)
 {
     return (voidpf) safe_emalloc(items, size, 0);
@@ -211,6 +262,18 @@ static sw_inline void php_zlib_free(voidpf opaque, voidpf address)
 }
 #endif
 
+#ifdef SW_HAVE_BROTLI
+static sw_inline void* php_brotli_alloc(void* opaque, size_t size)
+{
+    return emalloc(size);
+}
+
+static sw_inline void php_brotli_free(void* opaque, void* address)
+{
+    efree(address);
+}
+#endif
+
 static int http_parse_set_cookies(const char *at, size_t length, zval *cookies, zval *zset_cookie_headers)
 {
     const char *key = at;
@@ -261,6 +324,7 @@ bool swoole_websocket_handshake(http_context *ctx);
 
 #ifdef SW_USE_HTTP2
 int swoole_http2_server_onFrame(swServer *serv, swConnection *conn, swEventData *req);
+int swoole_http2_server_parse(http2_session *client, const char *buf);
 int swoole_http2_server_do_response(http_context *ctx, swString *body);
 void swoole_http2_server_session_free(swConnection *conn);
 void swoole_http2_response_end(http_context *ctx, zval *zdata, zval *return_value);
