@@ -33,22 +33,12 @@ extern "C" {
 #include <zlib.h>
 #endif
 
-enum http_client_state
-{
-    HTTP_CLIENT_STATE_WAIT,
-    HTTP_CLIENT_STATE_READY,
-    HTTP_CLIENT_STATE_BUSY,
-    //WebSocket
-    HTTP_CLIENT_STATE_UPGRADE,
-    HTTP_CLIENT_STATE_WAIT_CLOSE,
-    HTTP_CLIENT_STATE_CLOSED,
-};
-
 enum http_client_error_status_code
 {
     HTTP_CLIENT_ESTATUS_CONNECT_FAILED = -1,
     HTTP_CLIENT_ESTATUS_REQUEST_TIMEOUT = -2,
     HTTP_CLIENT_ESTATUS_SERVER_RESET = -3,
+    HTTP_CLIENT_ESTATUS_SEND_FAILED = -4,
 };
 
 enum http_client_error_flags
@@ -57,17 +47,6 @@ enum http_client_error_flags
     HTTP_CLIENT_EFLAG_UPGRADE = 1 << 1,
 };
 
-#ifdef SW_COROUTINE
-typedef enum
-{
-    HTTP_CLIENT_STATE_DEFER_INIT,
-    HTTP_CLIENT_STATE_DEFER_SEND,
-    HTTP_CLIENT_STATE_DEFER_WAIT,
-    HTTP_CLIENT_STATE_DEFER_DONE,
-} http_client_defer_state;
-#endif
-
-
 static sw_inline void http_client_create_token(int length, char *buf)
 {
     char characters[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!\"§$%&/()=[]{}";
@@ -80,24 +59,6 @@ static sw_inline void http_client_create_token(int length, char *buf)
     buf[length] = '\0';
 }
 
-static sw_inline int http_client_check_data(zval *data)
-{
-    if (Z_TYPE_P(data) != IS_ARRAY && Z_TYPE_P(data) != IS_STRING)
-    {
-        swoole_php_error(E_WARNING, "parameter $data must be an array or string");
-        return SW_ERR;
-    }
-    else if (Z_TYPE_P(data) == IS_ARRAY && php_swoole_array_length(data) == 0)
-    {
-        swoole_php_error(E_WARNING, "parameter $data is empty");
-    }
-    else if (Z_TYPE_P(data) == IS_STRING && Z_STRLEN_P(data) == 0)
-    {
-        swoole_php_error(E_WARNING, "parameter $data is empty");
-    }
-    return SW_OK;
-}
-
 static sw_inline void http_client_swString_append_headers(swString* swStr, const char* key, size_t key_len, const char* data, size_t data_len)
 {
     swString_append_ptr(swStr, key, key_len);
@@ -109,11 +70,11 @@ static sw_inline void http_client_swString_append_headers(swString* swStr, const
 static sw_inline void http_client_append_content_length(swString* buf, int length)
 {
     char content_length_str[32];
-    int n = snprintf(content_length_str, sizeof(content_length_str), "Content-Length: %d\r\n\r\n", length);
+    int n = snprintf(SW_STRS(content_length_str), "Content-Length: %d\r\n\r\n", length);
     swString_append_ptr(buf, content_length_str, n);
 }
 
-#ifdef SW_HAVE_ZLIB
+#ifdef SW_HAVE_COMPRESSION
 extern swString *swoole_zlib_buffer;
 #endif
 
