@@ -102,6 +102,7 @@ static const char *method_strings[] =
   , "NOTIFY"
   , "SUBSCRIBE"
   , "UNSUBSCRIBE"
+  , "PURGE"
   , "NOTIMPLEMENTED"
   };
 
@@ -520,7 +521,7 @@ size_t swoole_http_parser_execute (swoole_http_parser *parser,
           case 'M': parser->method = PHP_HTTP_MKCOL; /* or MOVE, MKCALENDAR, MKACTIVITY, MERGE, M-SEARCH */ break;
           case 'N': parser->method = PHP_HTTP_NOTIFY; break;
           case 'O': parser->method = PHP_HTTP_OPTIONS; break;
-          case 'P': parser->method = PHP_HTTP_POST; /* or PROPFIND or PROPPATCH or PUT */ break;
+          case 'P': parser->method = PHP_HTTP_POST; /* or PROPFIND, PROPPATCH, PUT, PATCH, PURGE */ break;
           case 'R': parser->method = PHP_HTTP_REPORT; break;
           case 'S': parser->method = PHP_HTTP_SUBSCRIBE; /* or SEARCH */ break;
           case 'T': parser->method = PHP_HTTP_TRACE; break;
@@ -569,13 +570,15 @@ size_t swoole_http_parser_execute (swoole_http_parser *parser,
         } else if (index == 1 && parser->method == PHP_HTTP_POST && ch == 'R') {
           parser->method = PHP_HTTP_PROPFIND; /* or HTTP_PROPPATCH */
         } else if (index == 1 && parser->method == PHP_HTTP_POST && ch == 'U') {
-          parser->method = PHP_HTTP_PUT;
+          parser->method = PHP_HTTP_PUT; /* or PHP_HTTP_PURGE */
         } else if (index == 1 && parser->method == PHP_HTTP_POST && ch == 'A') {
           parser->method = PHP_HTTP_PATCH;
         } else if (index == 1 && parser->method == PHP_HTTP_SUBSCRIBE && ch == 'E') {
           parser->method = PHP_HTTP_SEARCH;
         } else if (index == 2 && parser->method == PHP_HTTP_UNLOCK && ch == 'S') {
           parser->method = PHP_HTTP_UNSUBSCRIBE;
+        } else if (index == 2 && parser->method == PHP_HTTP_PUT && ch == 'R') {
+          parser->method = PHP_HTTP_PURGE;
         } else if (index == 4 && parser->method == PHP_HTTP_PROPFIND && ch == 'P') {
           parser->method = PHP_HTTP_PROPPATCH;
         } else {
